// © Copyright Mocoolka Corporation 2015,2017.
// Node module: mocoolka-tools
// LICENSE: MIT

/** @module mocoolka-tools-string */
const md5Source = require('md5');
const memoize = require('lodash/memoize');
const util = require('util');
const _string = require('underscore.string');

/**
 * convert message using md5
 * @param {string|buffer|Array}message
 * @param {Object}options encoding:'binary' message type is binary
 * asBytes:true return binary,asString:true return binary to string
 * @return {string|binary}
 */
const md5 = (message, options)=> (
  md5Source(message, options)
);

/**
 * cache md5 result value
 * @param {string|buffer|Array}message
 * @param {Object}options encoding:'binary' message type is binary
 * asBytes:true return binary,asString:true return binary to string
 * @return {string|binary}
 */
const md5Cache = (message, options)=> (
   (memoize(md5))(message, options)
);

/**
 * format message using variable
 * @param {...string|number|json} args
 * @return {string}
 */
const format = (...args)=>(
  util.format.apply(null, args)
);






/**
 * Converts underscored or dasherized string to a camelized one.
 * Begins with a lower case .
 *  ```js
 * stringTools.camelize('moz-transform');
 * //=> "mozTransform"
 *
 * stringTools.camelize('-moz-transform');
 * // => "mozTransform"
 *
 * stringTools.camelize('_moz_transform');
 * // => "mozTransform"
 *
 * stringTools.camelize('Moz-transform');
 * // => "mozTransform"
 *
 * ```
 * @param {string} str
 * @param {string} str
 * @return {string|*}
 */
const camelize = (str)=> {
  str = _string.camelize(str);
  str = str.slice(0, 1).toLowerCase() + str.slice(1);
  return str;
};
/**
 * Converts string to camelized class name. First letter is always upper case
 * ```js
 * stringTools.classify('-moz-transform');
 * => "MozTransform"
 * ```
 * @param {string} str the value be converted
 * @return {string} convert result value
 */
const classify = (str)=> {
  return _string.classify(str);
};
/**
 *Transform text into an ascii slug which can be used in safely in URLs.
 * Replaces whitespaces, accentuated, and special characters with a dash.
 * Limited set of non-ascii characters are transformed to similar versions in the ascii character set such as ä to a.
 * @param {string} str the value be converted to url
 * @returns {*}
 */
const urlSafe = (str)=> {
  return _string.slugify(str.toLowerCase().replace(/\./g, '-'));
};

/**
 * Converts a camelized or dasherized string into an underscored one
 * ```js
 * stringTools.underscored('MozTransform');
 * => "moz_transform"
 * ```
 * @param {string} str the value be converted
 * @return {string} convert result value
 */
const underscored = (str)=> {
  return _string.underscored(str);
};
/**
 * Converts a underscored or camelized string into an dasherized one
 * ```js
 * stringTools.dasherize('MozTransform');
 * => "-moz-transform"
 * ```
 * @param {string} str the value be converted
 * @return {string} convert result value
 */
const dasherize = (str)=> {
  return _string.dasherize(str);
};

const stringTools = {
  md5,
  md5Cache,
  format,
  camelize,
  classify,
  underscored,
  dasherize,
  urlSafe,
};

module.exports.__proto__ = stringTools;
