

/**
 * Module dependencies.
 */

const EventEmitter = require('events');
const debug = require('debug')('task-emitter');
const assert = require('assert');
const Task = require('./task');


/**
 * TaskEmitter
 */
class TaskEmitter extends EventEmitter{
  /**
   * Create a new `TaskEmitter`.
   */
  constructor(options) {
    super();
    this.tasks = [];
  }
  /**
   * Execute a task and emit an event when it is complete.
   */
  task (taskOptions) {
    let scope;
    let tasks = this.tasks;

    if (this.complete) {
      throw new Error('Could not start task '
        + name + '. Cannot add tasks after a TaskEmitter emits the done event.');
    }

    let task =  new Task(taskOptions);

    task.once('done', function (...doneArgs) {
      let args = {};
      args.input = [];
      args.output = [];
      let taskArgs = task.options.args;
      let taskName = task.options.name;

      args.input = taskArgs;
      args.output= doneArgs;

      this.emit(task.options.name, args);

      task.cleanup();

      // do any pending tasks remain
      // after a complete tick?
      process.nextTick(function () {
        let remaining = this.remaining();
        this.emit('progress', { remaining: remaining, total: this.tasks.length, task: taskName });

        if (!(remaining || this.complete)) {
          this.complete = true;
          this.emit('done');
        }
      }.bind(this));
    }.bind(this));

    task.once('error', function (err) {
      // stop all pending tasks
      this.stop();

      // emit the error
      this.emit('error', err);
    }.bind(this));

    tasks.push(task);

    task.run();

    return this;
  }
  /**
   * Stop all remaining tasks.
   *
   * @return {number}
   */
  stop() {
    this
      .tasks
      .forEach(function (t) {
        t.stop();
      });
  }
  /**
   * Determine how many tasks remain.
   *
   * @return {number}
   */
  remaining () {
    return this
      .tasks
      .map(function (t) {
        return t.complete ? 0 : 1;
      })
      .reduce(function (a, b) {
        return a + b;
      }, 0);
  }
  /**
   * Remove all tasks and reset all state.
   *
   * @return {number}
   */
  reset() {
    this.tasks = [];
    this.complete = false;
    this.removeAllListeners();
  };
}
const schemas = {
  task: {
    properties: {
      name: {
        type: 'string',
      },
      fn: {
        type: 'function',
      },
      scope: {
        type: 'object',
      },

    },
  },
};
/**
 * Expose `TaskEmitter`.
 */

module.exports = TaskEmitter;
