

/**
 * Module dependencies.
 */

const EventEmitter = require('events');
const debug = require('debug')('task');
const assert = require('assert');
const fs = require('fs');
const errorTools = require('../error-tools');
const typeTools = require('../type-tools');
/**
 * Task extends from `EventEmitter`.
 */
class Task extends EventEmitter{
  /**
   * Create a new `Task` with the given `options`.
   *
   * @param {Object} options
   * @return {Task}
   */
  constructor(options) {
    options = options || {};

    super();

    errorTools.validateJsonSchema
    ({ schema: schemas.init, data: options });
    if (options.name && options.scope && typeTools.isEmpty(options.fn)) {
      options.fn = options.scope[options.name];
    }

    options.scope = options.scope || {};
    this.options = options;
    this.complete = false;
    this.errored = false;
  }

  run() {
    let opts = this.options;
    let _this = this;
    let args = [];

    for (let i = 0; i < opts.args.length; i++) {
      args[i] = opts.args[i];
    }

    debug('%s - %s', opts.name, args.toString());

    // add complete callback
    args[args.length] = function () {
      _this._callback.apply(_this, arguments);
    };

    try {
      opts.fn.apply(opts.scope, args);
    } catch (e) {
      debug('%s errored!', opts.name);
      this.complete = this.errored = true;
      this.emit('error', e);
    }
  }
  /*!
   * Internal callback
   */
  _callback(err, ...otherArgs) {
    let args = [];

    // use diff signature for fs.exists
    if (this.options.scope == fs && this.options.name === 'exists') {
      args = [null, err];
      err = null;
    }

    // if the task has stopped
    // don't callback
    if (this.stopped) {
      return;
    }

    let opts = this.options;
    args = opts.args;

    this.complete = true;
    if (err) {
      this.errored = true;
      debug('ERROR - %s - %s - err: %j', opts.name, args.toString(), err);
      this.emit('error', err);
    } else {
      // remove error
      let args = otherArgs;

      debug('%s - %s - complete', opts.name, opts.args.toString());

      // add done to emit the done event
      // and pass along the args
      args.unshift('done');

      this.emit.apply(this, args);
    }
  }
  /**
   * Stop the current task.
   */
  stop () {
    this.stopped = true;
    this.cleanup();
  }
  /**
   * Remove any potential references to callbacks to prevent leaks.
   */
  cleanup  () {
    if (this.options) {
      delete this.options.fn;
      delete this.options.scope;
      delete this.options;
      this.options = {};
    }
  }
}

const schemas = {
  init: {
    properties: {
      name: {
        type: 'string',
      },
      fn: {
        type: 'function',
      },
      scope: {
        type: 'object',
      },
      args: {
        type: 'array',
      },

    },
    required: ['name'],
  },
};

/**
 * Expose `Task`.
 */

module.exports.__proto__ = Task;

